// PCF2219_driver_LCDI2C.C

// Programme de dmonstration de pilotage d'un afficheur LCD I2C BATRON "BTHQ 21605 AV-03"
// Le circuit driver est un PCF2119 de PHILIPS
// Le programme est ecrit pour un PIC 16F876A

//!!!!!!!!!!!!!!!!!!!!!!!!!!!       DEFINITION DES FONCTIONS         !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//                                                                                                                                                 !
// void Init_Ecran(void)                                       Initialisation du LCD (affiche des triangles)                                       !
// void Fixe_CT_DDRAM(int adrddram)                            Fixe l'adresse de la DDRAM a la valeur de adddram (contenu DDRAM inchange)          !
// void Lcd_Place_Curseur(int ligne, int colonne);             Deplace le curseur a la position fixee par ligne et colonne ( voir tableau )        !
//                                                             (contenu DDRAM inchange)                                                            !
// void Efface_Ecran(void);                                    Effacement de l'ecran avec position curseur en haut a gauche                        !
// void LCD_Retour_Maison();                                   Retour curseur en haut a gauche (contenu DDRAM inchange)                            !
// void Lcd_Decale_Curseur();                                  Deplace le curseur a droite(contenu DDRAM inchange)                                 !
// void Affiche_caractere(char car);                           Affichage d'un caractere a l'emplacement du curseur                                 !
//                                                                      \f  Effacement de l'ecran                                                  !
//                                                                      \n  Curseur en debut de 2e ligne                                           !
//                                                                      \b  Deplace le curseur a droite                                            !
// void Lcd_Print_nombre(unsigned int16 , unsigned char B)     Affichage d'un nombre a l'emplacement du curseur                                    !
//                                                                                                                                                 !
// L'utilisation de printf est permise                                                                                                             !
//                                                                                                                                                 !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


//!!!!!!!!!!!!!!!!!!!!!!!!!!!                TRAME I2C  D'ECRITURE   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//                                                                                                                                                 !
//    C0=1 ==> un autre octet de controle peut suivre    C0=0 ==> dernier octet de controle                                                        !
//    RS=0 ==> selection du registre d'instructions      RS=1 ==> selection du registre de donnees                                                 !
//    SA0 =1                                                                                                                                       !
//                                                                                                                                                 !
// !!!!!!!!! !!!!!!!!!!!!!!!! !!!!! !!!!!!!!!!!!!!! !!!!! !!!!!!!!!!!!!!! !!!!! !!!!!!!!!!!!!!! !!!!!!!!!!!!!!! !!!!! !!!!!       !!!!!!!!         !
// !       ! !        S  R  ! ! A ! !  C R        ! ! A ! !             ! ! A ! !  C R        ! !             ! ! A ! !   !       !      !         !
// ! Start ! ! 011101 A  W  ! ! C ! !  0 S 000000 ! ! C ! !  dddddddd   ! ! C ! !  0 S cccccc ! !  dddddddd   ! ! C ! ! P !       ! Stop !         !
// !       ! !        0 (0) ! ! K ! ! (1)         ! ! K ! !             ! ! K ! ! (0)         ! !             ! ! K ! !   !       !      !         !
// !!!!!!!!! !!!!!!!!!!!!!!!! !!! ! !!!!!!!!!!!!!!! !!!!! !!!!!!!!!!!!!!! !!!!! !!!!!!!!!!!!!!! !!!!!!!!!!!!!!! !!!!! !!!!!       !!!!!!!!         !
//                                                                                                                                                 !
//           ! Slave Adress !       ! Octet de    !       ! Octets de   !       ! Octet de    ! ! Octets de   !       ! Update !                   !
//           !              !       ! Controle de !       ! donnee/     !       ! Controle de ! ! donnee/     !       ! data   !                   !
//           !              !       ! registre    !       ! instruction !       ! registre    ! ! instruction !       ! pointer!                   !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                   !
//                                  ! <---------- X fois -------------> !                       ! <- x fois ->!                                    !
//                                                        !<--x fois--> !                                                                          !
//                                                                                                                                                 !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

//!!!!!!!!!!!!!!!!!!!!!!!!!!!       DEFINITION DES EMPLACEMENTS DES CARACTERES     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//                                                                                                                                                 !
//    Ligne1   Colonne           |1 |2 |3 |4 |5 |.....|16|           Ligne2   Colonne           |1 |2 |3 |4 |5 |.....|16|                          !
//             Adresse DDRAM     |0 |1 |2 |3 |4 |.....|15|                    Adresse DDRAM     |40|41|42|43|44|.....|79|                          !
//             en Hexa                                                        en Hexa                                                              !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


#define LCDADDR 0x76                   // adresse de base de l'afficheur LCD

//!!!!!!!!!!!!!!!!!!!!!!!!!!!       definition des prototypes     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Init_Ecran(void);                                // initialisation afficheur
void Efface_Ecran(void);                              // Effacement de l'ecran avec position curseur en haut a gauche
void LCD_Retour_Maison(void);                         // Retour curseur en haut a gauche ( contenu DDRAM inchange )
void Fixe_CT_DDRAM(int adrddram);                     // Fixe l'adresse de la DDRAM a la valeur de adrddram
void Lcd_Place_Curseur(int ligne, int colonne);       // Deplace le curseur a la position fixee par ligne et colonne ( voir tableau )
void Lcd_Decale_Curseur(void);                        // Deplace le curseur a droite(contenu DDRAM inchange)
void Affiche_caractere(char car);                     // Ecrit un caractere a l'adresse en cours  de DDRRAM
// Fin de definition des prototypes!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!Initialisation du LCD!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Init_Ecran(void)
{
   // Demarrage I2C
   i2c_start();
   // Envoi de l'adresse afficheur
   i2c_write(LCDADDR);  // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   // Envoi de l'octet de controle de slection de registre RS ( instruction)
   i2c_write(0x00);     // Octet de controle pour instruction     0b0000 0000  C0=0=>dernier octet de controle    RS=0=>next octet=commande
   // Definition de l'afficheur et de la gamme des instructions suivantes
   //i2c_write(0x35);   // Function_Set  0b001 1 0 101  DL=1=>interface 8 bits, M=1=>2*16   SL=0=>1:18   H=1=> instruction  etendue   i2c_write(0x35);     // Function_Set  0b0011 0100  DL=1=>interface 8 bits, M=1=>2*16   SL=0=>1:18   H=1=> instruction  etendue
   i2c_write(0x37);     // Function_Set  0b0011 0100  DL=1=>interface 8 bits, M=1=>2*16   SL=1=>1:9   H=1=> instruction  etendue

//!!!!!!!!!!!!!!!!!!!!!!!!!!!  instructions etendues   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   // Configuration des connections de l'ecran
   i2c_write(0x02);     // Screen_Config   0b0000 0010  L=0=>Connection standard des 2 moities de l'ecran
   // Configuration de miroir de l'afficheur
   i2c_write(0x08);     // Disp_Config   0b0000 01 00   P0=>Affichage colone 1 a 80  Q=0=>Affichage ligne 1 a 16 (pas de miroir vertical et horizontal)
   // Configuration des icones
   //i2c_write(0x08);   // Icon_Ctl 0b0000 1 000    IM=0=>mode caractere   IB=0=>non clignotement des icones  DM=0=>Pas de Mode direct
   // Configuration de la temperature
   i2c_write(0x10);     // Temp_Ctl 0b000100 00    TC1,TC2=00=> coefficient de temperature de VLCD=-0,16% /K (coef standard, valeur par defaut)
   // Multiplieur de tension
   i2c_write(0x42);     // HV_Gen        0b0100 0000  S1S0=00=> Etage multiplicateur de VLCD=1 (multiplieur=*2)
   //VLCD et registre
   i2c_write(0x9f);     // VLCD_Set      0b1 001 0000  V=0=>store to VA       facteur VA=16=>VLCD=3,10V

   // Definition de l'afficheur et de la gamme des instructions suivantes
   i2c_write(0x34);     // Function_Set  0b0011 0100  DL=1=>interface 8 bits, M=1=>2*16   SL=0=>1:18   H=0=> instruction  standard
//!!!!!!!!!!!!!!!!!!!!!!!!!!!  instructions standard   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   // Mise en marche afficheur et de la gamme des instructions suivantes
   //i2c_write(0x0C);      // Display_Ctl   0b0000 1 100     D=1=>Display on   C=0>curser off    B=0=>clignotement off
   i2c_write(0x0F);        // Display_Ctl   0b0000 1 111     D=1=>Display on   C=1>curser on    B=1=>clignotement on
   // Configuration de l'entree de donnee
   i2c_write(0x06);        // Entry_Mode_Set  0b0000 01 10    I/D=1=>incrementation de DDRAM  S=0=>Pas de dcalage de l'cran
   // Adresse de DDRAM
   i2c_write(0x80);        // Set_DDRAM     0b10000000   l'adresse de la DDRAM est 0b0000000 donc 0h00=0
   // Return Home
   i2c_write(0x02);        // Return_Home   0b00000010   Curseur en haut  gauche
   // Effacement Ecran
   i2c_write(0x01);        // Clear_Diplay  0b00000001   Effacement de l'ecran, reinitialsistion DDRAM
   delay_us(15000);
   // Stop I2C
   i2c_stop();
   delay_ms(1000);                  // attente 1s pour montrer mise en marche de l'afficheur
}
//Fin de Initialisation du LCD!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!!!     Effacement de l'ecran      Le jeu de caractere est R (voir doc)!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Efface_Ecran(void)
{
   int i;
   i2c_start();
   i2c_write(LCDADDR);  // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   i2c_write(0x80);     // Octet de controle pour instruction     0b1000 0000  C0=1=>octet de controle possible   RS=0=>next octet=commande
   i2c_write(0x80);     // 0b1 0000000    Adresse de DDRAM=0   (debut de ligne1)
   i2c_write(0x40);     // Octet de controle pour donnee          0b0100 0000  C0=0=>dernier octet de controle   RS=1=>next octet=donnee
   for (i=1;i<=16;i++)
      {
         i2c_write(0xA0);      // Effacement de la ligne 1
      }
   i2c_stop();
   i2c_start();
   i2c_write(LCDADDR);  // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   i2c_write(0x80);     // Octet de controle pour instruction     0b1000 0000  C0=1=>octet de controle possible   RS=0=>next octet=commande
   i2c_write(0xC0);     // 0b1 1000000       Adresse de DDRAM=64  (debut de ligne2)
   i2c_write(0x40);     // Octet de controle pour donnee          0b0100 0000  C0=0=>dernier octet de controle   RS=1=>next octet=donnee

   for (i=1;i<=16;i++)
      {
         i2c_write(0xA0);      // Effacement de la ligne 2
      }
    i2c_stop();
   Fixe_CT_DDRAM(0);
}
// Fin de Effacement de l'ecran      Le jeu de caractere est R (voir doc)!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!!!!!!! LCD Retour_Maison    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void LCD_Retour_Maison(void)           // Retour curseur en haut a gauche ( contenu DDRAM inchange )
{
   i2c_start();
   i2c_write(LCDADDR);           // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   i2c_write(0x00);              // Octet de controle pour instruction   0b0100 0000  C0=0=>dernier octet de controle   RS=0=>next octet=instruction
   i2c_write(0x02);              // Return_Home  0b0000 0010
   i2c_stop();
}
// Fin de LCD Retour_Maison!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!!!!!!! Lcd_Decale_Curseur    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Lcd_Decale_Curseur(void)                  // Deplace le curseur a droite(contenu DDRAM inchange)
{
   i2c_start();
   i2c_write(LCDADDR);           // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   i2c_write(0x00);              // Octet de controle pour instruction   0b0100 0000  C0=0=>dernier octet de controle   RS=0=>next octet=instruction
   i2c_write(0x14);              // Curs_Disp_Shift 0b0001 0100 SC=0=>move cursor   RL=0=>a droite
   i2c_stop();
}
// Fin de Lcd_Cursor_display_shift!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


// !!!!!!!!!!!!!!!!!!!!!!!!!! Valeur de DDRAM!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Fixe_CT_DDRAM(int adrddram)        // Fixe l'adresse de la DDRAM a la valeur de adddram
{
   i2c_start();
   i2c_write(LCDADDR);           // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
   i2c_write(0x00);              // Octet de controle pour instruction   0b0100 0000  C0=0=>dernier octet de controle   RS=0=>next octet=instruction
   i2c_write(128+adrddram);      // Set_DDRAM 128+Adresse de DDRAM          Adresse de DDRAM=adrddram
   i2c_stop();
}
// Fin de Valeur de DDRAM!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!!!!!!! Lcd_Place_Curseur  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Lcd_Place_Curseur (int ligne, int colonne)       // Deplace le curseur a la position fixee par ligne et colonne ( voir tableau )
{
   int Adr_pos,i;
      if (ligne==1)
         Adr_pos=colonne-1;
      else
         Adr_pos=colonne-1+0x40;
   Fixe_CT_DDRAM(Adr_pos);
}
// Fin de Lcd_Place_Curseur  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!!! Affichage d'un caractere a l'emplacement du curseur     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Affiche_caractere(char car)
{
   switch (car)
   {
      case '\f':
         Efface_Ecran();
         delay_ms(2);
         break;
      case '\n':
         Lcd_Place_Curseur(2,1);
         break;
      case '\b':
         Lcd_Decale_Curseur();
         break;
      default:
         {
            i2c_start();
            i2c_write(LCDADDR);        // Adresse du LCD   0x76=0b0111 01 10   SA0=1    R/W=0=>ecriture
            i2c_write(0x40);           // Octet de controle pour donnee   0b0100 0000  C0=0=>dernier octet de controle   RS=1=>next octet=donnee
            i2c_write(128+car);        // Write_Data         Affichage du caractere car
            i2c_stop();
         }
         break;
   }
}
// Fin de Affichage d'un caractere a l'emplacement du curseur       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

// !!!!!!!!!!!!!!!!!!!!! Affichage d'un nombre a l'emplacement du curseur   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void Lcd_Print_nombre(unsigned int16 NB, unsigned char B)
/* crit le nombre nb  partir de la position du curseur dans la base B
Entre 1: Le nombre  afficher.
Entre 2: La base dans laquelle le nombre doit tre crit.
			2 pour le binaire
			10 pour le dcimal			                                          */
{
	unsigned int16 chiffre,ancien,i,j;
	chiffre=NB;
	for(i=0;chiffre>=B;i++) chiffre=chiffre/B;
   Affiche_caractere(chiffre+48);
	while(i!=0)
	{
		ancien=NB;
		chiffre=NB;
		for(j=i;j!=0;j--) ancien=ancien/B;
		ancien=ancien*B;
		for(j=i-1;j!=0;j--) chiffre=chiffre/B;
		chiffre=chiffre-ancien;
      Affiche_caractere(chiffre+48);
		i--;
	}
}
// Fin de Affichage d'un nombre a l'emplacement du curseur   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
